# gaussian fits for linear ramps
setwd(base.dir)
source('M:/Niall/M026_scan_MASI_data_with_bg_correction.R')
source('A448_get_MASI_profile.R')
source('savitzky golay.R')
curr_date='180221'
current_directory=paste("Y:\\Documents\\Groups\\JMCP\\MASI\\Data\\MASI-",curr_date,sep='')

setwd(current_directory)
isos_files<-list.files(pattern="*processed.csv")
num_files=length(isos_files)

#.. Function for Euclidean distance
eucl.dist <- function(x, y) {
  sqrt(sum((x - y)^2))
}

#.. Function to use with optimise(). The first argument must be
#   the vector of parameters to search over.
#   data.x and data.y are the x and y co ordinates of the data.
norm.dist <- function(params, data.x, data.y) {
  mu <- params[1]
  sd <- params[2]
  scale <- params[3]
  gaussian <- dnorm(data.x, mu, sd, log = FALSE)*scale
  dist <- eucl.dist(data.y, gaussian)
  return(dist)
}

meas_key=scan("meas_key.txt",what="char",sep="\t")
meas_index=as.numeric(scan("meas_index.txt",what="char",sep="\t"))

stats_summary<-matrix(0,num_files,6)
pdf('M034_gaussian_fits_b.pdf')
for(j in 1:num_files){
    if(meas_key[j]=='m'){
        filename=isos_files[j]
        leng=nchar(filename)
        filename_original <- paste(substr(filename,1,leng-14),".csv",sep="")
        raw_data <- scan_MASI_data(current_directory,filename_original)
        
        profile<-get_MASI_profile(current_directory,filename)
        
        sodium <- raw_data[,1]
        metal2 <- raw_data[,2]/max(raw_data[,2])
        sodium_corr <- sodium
        metal2_corr <- metal2
        total_time <- max(raw_data[,3])
        time_expt <- raw_data[,3]/total_time
        measured_T <- raw_data[,4]
        if(max(raw_data[,3])>9){
            x<-time_expt[500:2500]-.5
            y <- sav.gol(metal2_corr[500:2500],15)
            plot(time_expt-.5,metal2_corr)
            points(x,y,col='blue')
            result <- optim(par = c(0.0, .01,1), fn = norm.dist,data.x = x, data.y = y)
            mu <- result$par[1]
            stdev <- result$par[2]
            scale <- result$par[3]
            y <- dnorm(x, mu, stdev, log = FALSE)*scale
            points(x,y,col='red',type='l')
            mu_kelvin <- (mu+.5)*total_time*120+1271
            sigma_kelvin <- stdev*total_time*120
            fwhm <- 2*sqrt(2*log(2))*sigma_kelvin
            stats_summary[j,]<-c((mu+.5)*total_time,stdev*total_time,scale,mu_kelvin,sigma_kelvin,fwhm)
            
            text(0,1, filename_original,cex=.7,pos=4)
            text(0,.9, stdev*total_time,cex=.7,pos=4)
            text(0,.8, j,cex=.7,pos=4,col='blue')
        } else {
            x<-time_expt
            y <- sav.gol(metal2_corr,15)
            plot(time_expt,metal2_corr)
            
            
            points(x,y,col='blue')
            result <- optim(par = c(0.0, .01,1), fn = norm.dist,data.x = x, data.y = y)
            mu <- result$par[1]
            stdev <- result$par[2]
            scale <- result$par[3]
            y <- dnorm(x, mu, stdev, log = FALSE)*scale
            points(x,y,col='red',type='l')
            mu_kelvin <- mu*total_time*120+1271
            sigma_kelvin <- stdev*total_time*120
            fwhm <- 2*sqrt(2*log(2))*sigma_kelvin
            stats_summary[j,]<-c(mu*total_time,stdev*total_time,scale,mu_kelvin,sigma_kelvin,fwhm)
            
            text(0,1, filename_original,cex=.7,pos=4)
            text(0,.9, stdev*total_time,cex=.7,pos=4)
            text(0,.8, j,cex=.7,pos=4,col='blue')
        }
    }
}
dev.off()
write.table(stats_summary,file='M034_stats_summary.txt',sep='\t')