# script to fit a gaussian function to LIF profile
# extracts fwhm and position of peak, in terms of time, altitude
source('M:/Niall/M026_scan_MASI_data_with_bg_correction.R')
source('M:/Analysis/A448_get_MASI_profile.R')
source('M:/Analysis/savitzky golay.R')
curr_date='171005'
current_directory=paste("Y:\\Documents\\Groups\\JMCP\\MASI\\Data\\MASI-",curr_date,sep='')

setwd(current_directory)
isos_files<-list.files(pattern="*processed.csv")
num_files=length(isos_files)

#.. Function for Euclidean distance
eucl.dist <- function(x, y) {
    sqrt(sum((x - y)^2))
}

#.. Function to use with optimise(). The first argument must be
#   the vector of parameters to search over.
#   data.x and data.y are the x and y co ordinates of the data.
norm.dist <- function(params, data.x, data.y) {
    mu <- params[1]
    sd <- params[2]
    scale <- params[3]
    gaussian <- dnorm(data.x, mu, sd, log = FALSE)*scale
    dist <- eucl.dist(data.y, gaussian)
    return(dist)
}

meas_key=scan("meas_key.txt",what="char",sep="\t")
meas_index=as.numeric(scan("meas_index.txt",what="char",sep="\t"))
setwd("M:\\Niall")
A<-scan("M040_cabmod_times_chergach.txt",what="char",sep="\t")
B<-as.numeric(A[22:length(A)])
C<-matrix(B,21,4401)
times_chergach=t(C)
A<-scan("M040_cabmod_times_allende.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
times_allende=t(C)
A<-scan("M040_cabmod_temperatures_chergach.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
temperatures_chergach=t(C)
A<-scan("M040_cabmod_temperatures_allende.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
temperatures_allende=t(C)


sizebin=rep('',num_files)
velocity=rep('',num_files)
stats_summary<-matrix(0,num_files,23)

setwd(current_directory)
pdf(paste('M041_gaussian_fits_',curr_date,'.pdf',sep=''))
for(j in 1:num_files){
    if(meas_key[j]=='m'){
        filename=isos_files[j]
        leng=nchar(filename)
        filename_original <- paste(substr(filename,1,leng-14),".csv",sep="")
        raw_data <- scan_MASI_data(current_directory,filename_original)
        
        profile<-get_MASI_profile(current_directory,filename)
        
        sodium <- raw_data[,1]
        metal2 <- raw_data[,2]/max(raw_data[,2])
        sodium_corr <- sodium
        metal2.smoothed <- sav.gol(metal2,105)
        total_time <- max(raw_data[,3])
        time_expt <- raw_data[,3]
        measured_T <- raw_data[,4]
        if(max(raw_data[,3])<9){
            
            meteorite<-substr(profile,20,27)
            if(meteorite=='chergach'){
                sizebin[j]=substr(profile,18,18) # need to sort this out
                velocity[j]=substr(profile,15,16) 
                
                chergach_cols<-c("x","v14c1","v14c2","v14c3","v14c4","x","v17c1","v17c2","v17c3","v17c4","x","v21c1","v21c2","v21c3",
                                 "v21c4","x","v31c1","v31c2","v31c3","v31c4","x")
                col_name<-substr(profile,14,18)
                col_index<-which(chergach_cols==col_name)
                setwd("Y:\\Documents\\Groups\\JMCP\\MASI\\Data\\profiles_171023")
                expt_profile<-substr(profile,14,31)
                expt_current<-scan(expt_profile)
                expt_max_index<-which.max(expt_current)
                expt_max_time <- expt_max_index*0.004    
                cabmod_max_index<-which.max(temperatures_chergach[,col_index])
                cabmod_max_time<-times_chergach[cabmod_max_index,col_index]
                
            } else { # i.e. allende
                sizebin[j]=substr(profile,12,12) 
                velocity[j]=substr(profile,9,10)   
                allende_cols<-c("x","v14c1","v14c2","v14c3","v14c4","x","v17c1","v17c2","v17c3","v17c4","x","v21c1","v21c2","v21c3",
                                "v21c4","x","v31c1","v31c2","v31c3","v31c4","x")
                col_name<-substr(profile,8,12)
                col_index<-which(allende_cols==col_name)
                setwd("Y:\\Documents\\Groups\\JMCP\\MASI\\Data\\profiles_171011")
                expt_profiles<-list.files(pattern="*.txt")
                expt_profile<-expt_profiles[grep(col_name,expt_profiles)]
                expt_current<-scan(expt_profile)
                expt_max_index<-which.max(expt_current)
                expt_max_time <- expt_max_index*0.004    
                cabmod_max_index<-which.max(temperatures_allende[,col_index])
                cabmod_max_time<-times_allende[cabmod_max_index,col_index]
            }
            
            hot_points_m2<-which(measured_T[200:1900]>1700)+199
            
            if(max(measured_T[200:1900])<1700){
                hot_points_m2<-which(measured_T[200:1900]>1400)+199
                
            } else if(max(measured_T[200:1900])<1600){
                hot_points_m2<-which(measured_T[200:1900]>1400)+199
                
            } else if(max(measured_T[200:1900])<1500){
                hot_points_m2<-which(measured_T[200:1900]>1400)+199
                
            }
            hot_point_first<-hot_points_m2[1]
            hot_point_end<-hot_points_m2[length(hot_points_m2)]
            hot_time_first<-time_expt[hot_point_first]
            hot_time_end<-time_expt[hot_point_end]
            hot_range<-seq(hot_point_first,hot_point_end)
            # scan in temperature compilations, select approprate column.
            # set tmax in order to match JDCS profile with experimental profile
            # that is, match maxima
            x<-(time_expt[hot_range]-hot_time_first)/(hot_time_end-hot_time_first)
            y <- metal2.smoothed[hot_range]
            plot(x,metal2[hot_range],type='l',col='red')
            
            total_time<-length(hot_range)/250
            points(x,y,col='blue',type='l')
            result <- optim(par = c(0.35, .01,1), fn = norm.dist,data.x = x, data.y = y)
            mu <- result$par[1]
            stdev <- result$par[2]
            scale <- result$par[3]
            y <- dnorm(x, mu, stdev, log = FALSE)*scale
            points(x,y,col='green',type='l')
            if(mu<0){mu<-0.01}
            if(mu>1){mu<-.99}
            mu_index <- round(mu*length(hot_range))
            mu_kelvin <- measured_T[hot_range[mu_index]]
            mu_time_expt <- time_expt[hot_range[mu_index]]
            mu_time_cabmod <- mu_time_expt - expt_max_time + cabmod_max_time
            sigma_time <- stdev*total_time
            fwhm_time <- 2*sqrt(2*log(2))*sigma_time
            start_fwhm_time <- mu_time_cabmod - fwhm_time/2
            finish_fwhm_time <- mu_time_cabmod + fwhm_time/2
            if(meteorite=='chergach'){
                start_fwhm_index <- which.min(abs(times_chergach[,col_index]-start_fwhm_time))
                finish_fwhm_index <- which.min(abs(times_chergach[,col_index]-finish_fwhm_time))
                mu_cabmod_index<- which.min(abs(times_chergach[,col_index]-mu_time_cabmod))
            }else{
                start_fwhm_index <- which.min(abs(times_allende[,col_index]-start_fwhm_time))
                finish_fwhm_index <- which.min(abs(times_allende[,col_index]-finish_fwhm_time))
                mu_cabmod_index<- which.min(abs(times_allende[,col_index]-mu_time_cabmod))             
            }
            
            #sigma_kelvin <- stdev*total_time*120
            #fwhm_kelvin <- 2*sqrt(2*log(2))*sigma_kelvin
            altitude <- seq(500,60,by=-0.1)
            fwhm_altitude<- altitude[start_fwhm_index]-altitude[finish_fwhm_index]
            mu_altitude <- altitude[mu_cabmod_index]
            text(0,1, filename_original,cex=.7,pos=4)
            text(0,.9, stdev*total_time,cex=.7,pos=4)
            text(0,.8, j,cex=.7,pos=4,col='blue')
            ###### simple fwhm from smoothed data #####
            half_max=max(metal2.smoothed)/2 
            
            peak_index=which(metal2.smoothed==max(metal2.smoothed))
            peak_time=time_expt[which(metal2.smoothed==max(metal2.smoothed))]
            peak_temp=measured_T[which(metal2.smoothed==max(metal2.smoothed))]
            # find the maximum and the corresponding time and temperature
            half_max_index_1=which.min(abs(metal2.smoothed[1:peak_index]-half_max))
            half_max_index_2=which.min(abs(metal2.smoothed[peak_index:length(time_expt)]-half_max))+peak_index
            
            T_half_max_1=measured_T[half_max_index_1]
            T_half_max_2=measured_T[half_max_index_2]
            T_fwhm=T_half_max_2-T_half_max_1
            time_half_max_1=time_expt[half_max_index_1]
            time_half_max_2=time_expt[half_max_index_2]
            time_fwhm_smoothed=time_half_max_2-time_half_max_1
            time_half_max_1_cabmod<-time_half_max_1 - expt_max_time + cabmod_max_time
            time_half_max_2_cabmod<-time_half_max_2 - expt_max_time + cabmod_max_time
            peak_time_cabmod<-peak_time - expt_max_time + cabmod_max_time
            if(meteorite=='chergach'){
                start_fwhm_index_smoothing <- which.min(abs(times_chergach[,col_index]-time_half_max_1_cabmod))
                finish_fwhm_index_smoothing  <- which.min(abs(times_chergach[,col_index]-time_half_max_2_cabmod))
                max_smoothing_index_cabmod  <- which.min(abs(times_chergach[,col_index]-peak_time_cabmod))
            }else{
                start_fwhm_index_smoothing  <- which.min(abs(times_allende[,col_index]-time_half_max_1_cabmod))
                finish_fwhm_index_smoothing  <- which.min(abs(times_allende[,col_index]-time_half_max_2_cabmod))
                max_smoothing_index_cabmod  <- which.min(abs(times_allende[,col_index]-peak_time_cabmod))
                
            }
            max_altitude <- altitude[max_smoothing_index_cabmod]
            fwhm_altitude_smoothing<- altitude[start_fwhm_index_smoothing]-altitude[finish_fwhm_index_smoothing]
            points(c(time_half_max_1,time_half_max_2),c(0,0),type='l',col='black')
            stats_summary[j,]<-c(mu_time_expt,sigma_time,scale,mu_kelvin,fwhm_time,
                                 mu_index,mu_time_cabmod,start_fwhm_time,finish_fwhm_time,
                                 start_fwhm_index,finish_fwhm_index,fwhm_altitude,
                                 half_max,fwhm_altitude_smoothing,T_half_max_1,T_half_max_2,
                                 time_half_max_1,time_half_max_2,time_fwhm_smoothed,mu_altitude,max_altitude,peak_temp,peak_time)
            points(x,measured_T[hot_range]/2700,type='l',col='magenta',lwd=2)
        }
    }
}
dev.off()
setwd(current_directory)
write.table(stats_summary,file='M041_stats_summary.txt',sep='\t')