# Plots 3x3 arrays of MASI and CABMOD atmospheric profiles, as function of time
# Includes error envelope (lower and upper limits for experimental data)
# base.dir<- "M:\\Niall"
base.dir<- "C:\\Users\\David\\Documents\\Niall"
#filename = "MASI-2017-02-22_12-56-14.csv"
# update of A620, A709 for nickel
# update of M116 - only plots CABMOD update

setwd(base.dir)
source("savitzky golay.R")
source('M026_scan_MASI_data_with_bg_correction.R')
#source('M:/Analysis/A695_DLB_legend_A694.R')

# M040 files are precompiled matrices of cabmod data (times and temperatures for all
# particle sizes) to speed up this script
A<-scan("M040_cabmod_times_chergach.txt",what="char",sep="\t")
B<-as.numeric(A[22:length(A)])
C<-matrix(B,21,4401)
times_chergach=t(C)
A<-scan("M040_cabmod_times_allende.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
times_allende=t(C)
A<-scan("M040_cabmod_temperatures_chergach.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
temperatures_chergach=t(C)
A<-scan("M040_cabmod_temperatures_allende.txt",what="char",sep="\t")
B<-as.numeric(A[26:length(A)])
C<-matrix(B,25,4401)
temperatures_allende=t(C)

setwd(base.dir)
A=scan("M118_M077_file_list_atmospheric_Ni_MASI.txt",what="char",sep="\t")
# file format:
# 1. directory
# 2. id number
# 3. file number
# 4. file names
# 5. velocity
# 6. particle size
# 7. meteorite name
# 8. heating profile file name
# 9. peak time Na
# 10. peak time Ni
# 11. fraction ablated Na
# 12. fraction ablated Ni

# v31 profiles removed
B<-matrix(A,12,length(A)/12)
D=t(B)
MASI_directory <- D[,1]
isos_number <- as.numeric(D[,3])
filename_list <- D[,4]
velocity <- as.numeric(D[,5])
sizes <- as.numeric(D[,6])
meteorite <- (D[,7])
meteorite<-c(meteorite,"stop") # so the loop knows when its at the last file.
sizes<-c(sizes,0) # so the loop knows when its at the last file.

profile_list <- D[,8]
peak_time_Na <- as.numeric(D[,9])
peak_time_Ni <- as.numeric(D[,10])
frac_Ni_ablated <- abs(as.numeric(D[,12]))
frac_Ni_ablated[which(frac_Ni_ablated>1)]<-1
frac_Na_ablated <- abs(as.numeric(D[,11]))
frac_Na_ablated[which(frac_Na_ablated>1)]<-1
# average peak_max times
setwd(base.dir)
A=scan("M169_M117_M068_peak_max_time_NaNi_MASI.txt",what="char",sep="\t")
# M117 also has fractions ablated from CABMOD
# and experimental std.dev.fraction ablated
# v31 profiles removed
B<-matrix(A,10,length(A)/10)
D=t(B)

avg_peak_time_Na <- as.numeric(D[,3])
avg_peak_time_Ni <- as.numeric(D[,5])
unit_sd_peak_na = round(as.numeric(D[,4])*10)
unit_sd_peak_ni = round(as.numeric(D[,6])*250)
unit_na_peak = round(avg_peak_time_Na*10)
unit_ni_peak = round(avg_peak_time_Ni*250)
frac_ablated_cabmod_na= as.numeric(D[,7])
frac_ablated_cabmod_ni= as.numeric(D[,8])
expt_sd_frac_ablated_na = as.numeric(D[,9])
expt_sd_frac_ablated_ni = as.numeric(D[,10])
smoothing_factor <- 151

mass_list_allende<-c('1Em2','6Em1','3Em0','1Ep1')# allende
mass_list_chergach <-c('1Em2','7Em1','4Em0','1Ep1')# chergach
#
colours <- c("#ffbb00fe","#00dd99fe","#ff7700fe","#009900fe","#00bbfffe","#0000ddfe","#000000fe","#444444fe",
"#ffdd88fe","#88eeccfe","#88ddfffe","#888888fe")
old_profile<-profile_list[31] # to be different from profile_list[1]
num_LIF_profiles <- length(profile_list)
profile_type<-0
averages_index <- 1
num_spectra=0
na_sum_spectra<-rep(0,119) # 2 seconds added at each end
ni_sum_spectra<-rep(0,2999) # 2 seconds added at each end

ni_all_spectra<-matrix(0,2999,15)
na_all_spectra<-matrix(0,119,15)

times=NULL
pdf("M183_M169_with_exptl_data.pdf")

par(mai=c( 0, 0, 0, 0),oma=c(5,5,2,2),lwd=1,cex=1.5,mfrow=c(3,3))
#for(j in 1:12){
for(j in 1:num_LIF_profiles){
    num_spectra<-num_spectra+1    
    current_meteorite=meteorite[j]
    current_size<-sizes[j]
    
    time_Na_peak <- peak_time_Na[j]
    curr_avg_Na_time <- avg_peak_time_Na[averages_index]
    time_Ni_peak <- peak_time_Ni[j]
    curr_avg_Ni_time <- avg_peak_time_Ni[averages_index]
    # get data for current file
    current_directory=paste(base.dir,"/MASI-",MASI_directory[j],sep="")
    setwd(current_directory)
    isos_files<-list.files(pattern="*.csv")
    filename <- isos_files[(isos_number[j]*2)-1]
    raw_data <- scan_MASI_data(current_directory,filename)
    
    filename <- isos_files[(isos_number[j]*2)]
    raw_data_Na <- scan_MASI_data(current_directory,filename)
    
    sodium<-raw_data_Na[,1]
    metal2<-raw_data[,2]
    measured_T <- raw_data[,4]
    
    time_corrected <- seq(.004,7.996,by=.004) #- time_Ni_peak + curr_avg_Ni_time
    shift_ni<- curr_avg_Ni_time - time_Ni_peak
    if(time_Na_peak!=0){
        
    shift_na<- curr_avg_Na_time - time_Na_peak
        
    } else {
    shift_na<-0
    }   

    sodium.smoothed <- sav.gol(sodium,6)
    scaling_na=1/max(sodium.smoothed)*frac_Na_ablated[j]
    sodium.scaled=sodium.smoothed*scaling_na
    
    metal2_smoothed <- sav.gol(metal2,smoothing_factor)
    scaling_M2=1/max(metal2_smoothed)*frac_Ni_ablated[j]
    metal2.scaled=metal2_smoothed*scaling_M2
    

    unit_shift_na=round(10*shift_na)
    na_start<-20+unit_shift_na
    na_finish<-na_start+78
    na_sum_spectra[na_start:na_finish]<-na_sum_spectra[na_start:na_finish]+sodium.scaled
    na_all_spectra[na_start:na_finish,num_spectra]<-sodium.scaled
    unit_shift_ni=round(250*shift_ni)
    ni_start<-500+unit_shift_ni
    ni_finish<-ni_start+1998
    ni_sum_spectra[ni_start:ni_finish]<-ni_sum_spectra[ni_start:ni_finish]+metal2.scaled
    ni_all_spectra[ni_start:ni_finish,num_spectra]<-metal2.scaled
    
    if((meteorite[j+1]!=current_meteorite)||(sizes[j+1]!=current_size)){
        profile_type <- profile_type + 1
        curr_profile <- profile_list[j]
        plot(1,1,type='n',xlim=c(1,3),ylim=c(1200,2800))
        if(meteorite[j]=='allende'){
        #setwd(paste("C:\\Users\\chmdbo\\Documents\\reference\\JDCS\\Atmospheric_profiles\\allende\\v",velocity[j],sep=""))
        setwd(paste("C:\\Users\\David\\Documents\\Niall\\JDCS ablation profiles with nickel\\Atmospheric profiles\\Allende\\v",velocity[j],sep=""))
          
        filename = paste("sza035v0",velocity[j],"mass",mass_list_allende[sizes[j]],"test_comp_allendev2.txt",sep="")
        } else if(meteorite[j]=='chergach'){
            #setwd(paste("C:\\Users\\chmdbo\\Documents\\reference\\JDCS\\Atmospheric_profiles\\chergach\\v",velocity[j],sep=""))
            setwd(paste("C:\\Users\\David\\Documents\\Niall\\JDCS ablation profiles with nickel\\Atmospheric profiles\\Chergach\\v",velocity[j],sep=""))
          
            filename = paste("sza035v0",velocity[j],"mass",mass_list_chergach[sizes[j]],"test_comp_chergach3.txt",sep="")
            
        }
        A=scan(filename,what="char",sep=",")
        data=A
        B=as.numeric(data)
        
        C<-matrix(B,50,length(B)/50)
        D=t(C)

        cabmod_Ni <- D[,48]
        cabmod_alt <- D[,1]
        cabmod_time <- D[,2]
        cabmod_temp <- D[,4]
        
        cabmod_Na <- D[,28]
        if(meteorite[j]=='chergach'){
            
            chergach_cols<-c("x","v14c1","v14c2","v14c3","v14c4","x","v17c1","v17c2","v17c3","v17c4","x","v21c1","v21c2","v21c3",
                             "v21c4","x","v31c1","v31c2","v31c3","v31c4","x")
            col_name<-substr(curr_profile,14,18)
            col_index<-which(chergach_cols==col_name)
            setwd(paste(base.dir,"\\profiles_171023",sep=""))
            expt_profile<-substr(curr_profile,14,31)
            expt_current<-scan(expt_profile)
            expt_max_index<-which.max(expt_current)
            expt_max_time <- expt_max_index*0.004    
   
        } else if(meteorite[j]=='allende'){ # i.e. allende
            
            allende_cols<-c("x","v14c1","v14c2","v14c3","v14c4","x","v17c1","v17c2","v17c3","v17c4","x","v21c1","v21c2","v21c3",
                            "v21c4","x","v31c1","v31c2","v31c3","v31c4","x")
            col_name<-substr(curr_profile,8,12)
            col_index<-which(allende_cols==col_name)
            setwd(paste(base.dir,"\\profiles_171011",sep=""))
            expt_profiles<-list.files(pattern="*.txt")
            expt_profile<-expt_profiles[grep(col_name,expt_profiles)]
            expt_current<-scan(expt_profile)
            expt_max_index<-which.max(expt_current)
            expt_max_time <- expt_max_index*0.004    
       
        }
        if(max(expt_current)>0.00628){
            find_start<-which(abs(expt_current-0.00627)<1e-4)
            time_start_index<-find_start[1]
            time_start<-time_start_index*.004
        } else{time_start<-4.176} # case for allende v14c1
        unit_time_start_na = round(10*time_start)
        unit_time_start_ni = round(250*time_start)
        cabmod_max_index<-which.max(cabmod_temp)
        cabmod_max_time<-cabmod_time[cabmod_max_index]
        cabmod_time_corr<-cabmod_time-cabmod_max_time+expt_max_time-time_start+1
        cabmod_frac_na<-frac_ablated_cabmod_na[averages_index]
        cabmod_frac_ni<-    frac_ablated_cabmod_ni[averages_index]
        time_corrected <- seq(.004,7.996,by=.004)# - time_start + 1
        time_corrected_Na <- seq(.1,7.9,by=.1)# - time_start + 1
        
        time_plot_start_na = which.min(abs(time_corrected_Na-.9))  
        time_plot_start_ni = which.min(abs(time_corrected-.9))
        time_plot_finish_na = which.min(abs(time_corrected_Na-3))  
        time_plot_finish_ni = which.min(abs(time_corrected-3)) 
        times=c(times,time_plot_finish_na)
        na_all_spectra[which(na_all_spectra< -0.048)]=-.048
        ni_all_spectra[which(ni_all_spectra< -0.048)]=-.048
        for(k in 1:num_spectra){
           points(time_corrected_Na[time_plot_start_na:time_plot_finish_na],na_all_spectra[20+(time_plot_start_na:time_plot_finish_na),k]*1500+1200,type='l',col='tan1',lwd=1) 
            points(time_corrected[time_plot_start_ni:time_plot_finish_ni],ni_all_spectra[500+(time_plot_start_ni:time_plot_finish_ni),k]*1500+1200,type='l',col='tan',lwd=1)            
        } 
        na_sum_spectra=na_sum_spectra/num_spectra
        ni_sum_spectra=ni_sum_spectra/num_spectra
        na_sum_spectra[which(na_sum_spectra< -0.048)]=NaN
        ni_sum_spectra[which(ni_sum_spectra< -0.048)]=NaN

        
        #points(time_corrected_Na[time_plot_start_na:time_plot_finish_na],na_sum_spectra[20+(time_plot_start_na:time_plot_finish_na)]*1500+1200,
        #        type='l',col='orange',lwd=2) 
        #points(time_corrected[time_plot_start_ni:time_plot_finish_ni],ni_sum_spectra[500+(time_plot_start_ni:time_plot_finish_ni)]*1500+1200,
        #       type='l',col='brown',lwd=2) 
        cabmod_start_index = which.min(abs(cabmod_time_corr-0.9))
        cabmod_finsh_index = which.min(abs(cabmod_time_corr-3))
        points(cabmod_time_corr[cabmod_start_index:cabmod_finsh_index],cabmod_Ni[cabmod_start_index:cabmod_finsh_index]/max(cabmod_Ni)*cabmod_frac_ni*1500+1200,type='l',lwd=2,
               col='brown',lty=2)
        
        points(cabmod_time_corr[cabmod_start_index:cabmod_finsh_index],cabmod_Na[cabmod_start_index:cabmod_finsh_index]/max(cabmod_Na)*cabmod_frac_na*1500+1200,type='l',lwd=2,
               col='orange',lty=2)
        #points(cabmod_time_corr[cabmod_start_index:cabmod_finsh_index],cabmod_temp[cabmod_start_index:cabmod_finsh_index],type='l',lwd=2,
        #       col='red')
   
        
        # plotting error envelope
        sd_current_ni=rep(0,2999)
        mean_current_ni=rep(0,2999)
        sd_current_na=rep(0,119)
        mean_current_na=rep(0,119)
        envelope_lower_na=rep(0,119)
        envelope_upper_na=rep(0,119)
        envelope_lower_ni=rep(0,2999)
        envelope_upper_ni=rep(0,2999)
        for(k in 1:2999){
            sd_current_ni[k]<-sd(ni_all_spectra[k,1:num_spectra])
            mean_current_ni[k]<-mean(ni_all_spectra[k,1:num_spectra])
        }
        for(k in 1:119){
            sd_current_na[k]<-sd(na_all_spectra[k,1:num_spectra])
            mean_current_na[k]<-mean(na_all_spectra[k,1:num_spectra])
            
        }
        # na envelope
        #for(k in 20:98){
        #    envelope_upper_na[k] = max(c(mean_current_na[k]+sd_current_na[k],mean_current_na[k]*(1+expt_sd_frac_ablated_na))) 
        #    envelope_lower_na[k] = min(c(mean_current_na[k]-sd_current_na[k],mean_current_na[k]*(1-expt_sd_frac_ablated_na))) 
        #}

        envelope_upper_na = sav.gol(envelope_upper_na,7)
        envelope_lower_na = sav.gol(envelope_lower_na,7)
        envelope_lower_na[which(envelope_lower_na< -0.048)]=-0.048
        envelope_upper_na[which(envelope_upper_na> 1.05)]=1.05
     
        #points(time_corrected_Na[time_plot_start_na:time_plot_finish_na],envelope_upper_na[20+(time_plot_start_na:time_plot_finish_na)]*1500+1200,type='l',col='blue')
        #points(time_corrected_Na[time_plot_start_na:time_plot_finish_na],envelope_lower_na[20+(time_plot_start_na:time_plot_finish_na)]*1500+1200,type='l',col='blue')
        # ni envelope
        for(k in 500:2498){
            envelope_upper_ni[k] = max(c(mean_current_ni[k]+sd_current_ni[k],mean_current_ni[k]*(1+expt_sd_frac_ablated_ni))) 
            envelope_lower_ni[k] = min(c(mean_current_ni[k]-sd_current_ni[k],mean_current_ni[k]*(1-expt_sd_frac_ablated_ni))) 
        }

        envelope_upper_ni = sav.gol(envelope_upper_ni,51)
        envelope_lower_ni = sav.gol(envelope_lower_ni,51)
        envelope_lower_ni[which(envelope_lower_ni< -0.048)]=-0.048
        envelope_upper_ni[which(envelope_upper_ni> 1.05)]=1.05
     
        #points(time_corrected[time_plot_start_ni:time_plot_finish_ni],envelope_upper_ni[500+(time_plot_start_ni:time_plot_finish_ni)]*1500+1200,type='l',col='darkgreen')
        #points(time_corrected[time_plot_start_ni:time_plot_finish_ni],envelope_lower_ni[500+(time_plot_start_ni:time_plot_finish_ni)]*1500+1200,type='l',col='darkgreen')  
        
        
        
        num_spectra<-0 #reset since next file is new size
        ni_all_spectra<-matrix(0,2999,15)
        na_all_spectra<-matrix(0,119,15)
        na_sum_spectra<-rep(0,119) # 2 seconds added at each end
        ni_sum_spectra<-rep(0,2999) # 2 seconds added at each end
        averages_index<-averages_index+1
       
    }
}
dev.off()

